# ========================================================================================================
# Mark Neeser
# 2015.07.01
#
# A python interactive window for use in the Austrian in-kind
# fringe-correction Reflex workflow.  This interactive window
# shows the results of the fringe-correction pipeline, allowing
# the User to compare the input image with the fringe-corrected
# image, and the fringe map.   From this interactive Actor the
# User can modify pipeline parameters and re-initiate the
# processing.
#
# Pipeline Parameters for inclusion:
#
# 1. mime_create_obj_mask
#    --sigma_det      object detection threshold in units of standard deviation [3.0]
#    --growing_radius radius by which mask of detected object should be grown (<= 100) [5]
#
# 2. mime_compute_fringe
#    --rescale        measure fringe amplitude and rescale images before stacking? [yes]
#    --nmin_comb      minimum number of jittered images to combine [1]
#    --method         method for computing a robust mean: median|min_max_rejection|
#                     kappa_sigma_clip|mean_median_ratio_constraint|average|sum) [median]
#    --rejlow         the number of low-level frames to reject [3]
#    --rejhigh        the number of high-level frames to reject [3]
#    --kappa_low      lower side rejection kappa multiple [3.0]
#    --kappa_high     high side rejection kappa multiple [3.0]
#    --ks_iter        number of iterations of kappa sigma clipping for running computation of mean [5]
#    --mmr_threshold  threshold for mean/median ratio [1.001]
#
# 3. mime_subtract_fringecor
#    --rescale        if "yes", then the fringe amplitude is measured in each input image and the fringe image
#                     is scaled accordingly before subtracting.  [yes]
#
#
# Relevant output images to plot (and the recipes that created them)
#
# 0. RAW              (the original input image)
# 1. OBJ_MASK         (mime_create_obj_mask)      ==> mask of sources in image
# 2. FRINGE_IMAGE     (mime_compute_fringecor)    ==> pure fringe map (no sources)
# 3. FRINGE_CORRECTED (mime_subtract_fringecor)   ==> fringe-corrected input image
#
#
# ========================================================================================================

from __future__ import absolute_import
import os
import sys
sys.path.append(os.path.dirname(__file__))
from esotk_fringe_gui import BaseDataPlotterManager, runInteractiveApp

try:
    # Import non-standard but needed modules. These must be as subset of what
    # the esotk_fringe_gui module tries to load.
    import reflex
    import pipeline_product
    import pipeline_display

    class DataPlotterManager(BaseDataPlotterManager):

        # This function will read all the columns, images and whatever is needed
        # from the products. The variables , self.plot_x, self.plot_y, etc...
        # are used later in function plotProductsGraphics().
        def readFitsData(self, fitsFiles):
            # Control variable to check if the interesting files were found in
            # the input.
            self.science_frames = []
            self.mask_frames = []

            # Read all the products
            for frame in fitsFiles:
                if frame == '':
                    continue
                category = frame.category
                if category == 'RAW':
                    self.science_frames.append(frame)
                elif category == 'OBJ_MASK':
                    self.mask_frames.append(frame)

            self.science_found = len(self.science_frames) > 0
            self.mask_found = len(self.mask_frames) > 0

            if self.science_found:
                frame = self.science_frames[0]
                self.science = pipeline_product.PipelineProduct(frame)
                self.setupExtensionValues(self.science)

            if self.mask_found:
                frame = self.mask_frames[0]
                self.mask = pipeline_product.PipelineProduct(frame)

        def loadProducts(self, index):
            """
            Loading both science and object masks.
            """
            super(DataPlotterManager, self).loadProducts(index)
            if index >= 0 and index < len(self.mask_frames):
                frame = self.mask_frames[index]
                self.mask = pipeline_product.PipelineProduct(frame)

        # This function creates all the subplots. It is responsible for the
        # plotting layouts.
        # There can different layouts, depending on the availability of data
        # Note that subplot(I,J,K) means the Kth plot in a IxJ grid
        # Note also that the last one is actually a box with text, no graphs.
        def addSubplots(self, figure):
            self.list_subplot_image     = []
            if self.science_found == True and self.mask_found == True:
                self.subplot_image1 = figure.add_subplot(1, 2, 1)
                self.subplot_image2 = figure.add_subplot(1, 2, 2)
            else:
                self.subtext_nodata = figure.add_subplot(1, 1, 1)
            self.setupExtensionLabel(figure)

        # This is the function that makes the plots.
        # Add new plots or delete them using the given scheme.
        # The data has been already stored in self.plot_x, self.plot_xdif, etc ...
        # It is mandatory to add a tooltip variable to each subplot.
        # One might be tempted to merge addSubplots() and plotProductsGraphics().
        # There is a reason not to do it: addSubplots() is called only once at
        # startup, while plotProductsGraphics() is called always there is a
        # resize.
        def plotProductsGraphics(self):

            if self.science_found == True and self.mask_found == True:

                # Display input image and masked image
                
                title_image1 = 'Input Image ext['+str(self.ext_id)+']'
                tooltip_image1 = "This is the input image with fringes"
                
                title_image2 = 'Object Mask ext['+str(self.ext_id)+']'
                tooltip_image2 = "This is the mask image to remove sources in the input"
                
                ext_id = self.ext_id
                self.science.readImage(fits_extension=ext_id)
                self.mask.readImage(fits_extension=ext_id)
                imgdisp = pipeline_display.ImageDisplay()
                imgdisp.setLabels('X (pixels)', 'Y (pixels)')
                imgdisp.setAspect('equal')  # this sets images to an equal aspect ratio

                # using a MAD to find reasonable display limits for the input image:
                image1 = self.science.image[ext_id]
                new_med, new_mad = self.computeRobustMedMad(image1)

                imgdisp.z_lim = new_med-1.48*new_mad, new_med+3*1.48*new_mad
                imgdisp.setCmap('hot')
                imgdisp.display(self.subplot_image1, title_image1, tooltip_image1,
                                self.science.image)
                
                self.subplot_image1.set_title(title_image1, fontsize=12, fontweight='semibold', rotation='vertical',x=1.05,y=0.6)
                self.subplot_image2.set_title(title_image2, fontsize=12, fontweight='semibold', rotation='vertical',x=1.05,y=0.6)

                # Set limits to mask image
                imgdisp.z_lim = -0.5, 1.5
                imgdisp.setCmap('hot')
                imgdisp.display(self.subplot_image2, title_image2, tooltip_image2,
                                self.mask.image)

                self.updateExtensionLabel()

            else:
                # Data not found info
                self.subtext_nodata.set_axis_off()
                self.text_nodata = """Data not found. Input files should contain these types:
RAW
OBJ_MASK
"""
                self.subtext_nodata.text(0.1, 0.6, self.text_nodata,
                                         color = '#11557c', fontsize = 18,
                                         ha = 'left', va = 'center', alpha = 1.0)
                self.subtext_nodata.tooltip = "Data not found in the products"

        # This function specifies which are the parameters that should be presented
        # in the window to be edited.
        # Note that the parameter has to be also in the in_sop port (otherwise it
        # won't appear in the window)
        # The descriptions are used to show a tooltip. They should match one to one
        # with the parameter list
        # Note also that parameters have to be prefixed by the 'recipe name:'
        def setInteractiveParameters(self):
            paramList = list()

            rec_id = "esotk_mask_create"
            group_id = ""

            pl = self.interactive_app.inputs.in_sop
            for p in pl:
                
                p_name=p.displayName
                if p_name == "obj.min-pixels" or p_name == "obj.threshold" or p_name == "obj.deblending" or p_name == "obj.core-radius":
                    group_id = "obj"
                elif p_name == "bkg.estimate" or p_name == "bkg.mesh-size" or p_name == "bkg.smooth-gauss-fwhm":
                    group_id = "bkg"
                elif p_name == "det.effective-gain" or p_name == "det.saturation":
                    group_id = "det"
                elif p_name == "pfx" or p_name == "pfy" or p_name == "pfm":
                    group_id = "post-filtering"
                
                p_desc=p.description

                if p.valenum :
                    p_desc += ' <'
                    for value in p.valenum :  
                        p_desc += value + ' |'
                    p_desc = p_desc[:-1] + "> [" +p.value + "]"
                
                paramList.append(reflex.RecipeParameter(
                     rec_id, p_name, group=group_id, description=p_desc))
                
            return paramList


        def setWindowHelp(self):
            help_text = """
This is an interactive window which help to assess the quality of the execution of a recipe.
"""
            return help_text

        def setWindowTitle(self):
            title = 'Pipeline Interactive Window'
            return title

except ImportError:
    DataPlotterManager = None


if __name__ == '__main__':
    runInteractiveApp(DataPlotterManager)
